% Compte-Rendu du TD : Probabilites conditionelles et Classification
% Auteur : Soizic Geslin et Maxime Chambreuil
% Date : 3  Fevrier 2003

close all
clear all

%%%
%%% INITIALISATIONS
%%%

heart = load('heart.data');
N = length(heart);
nbApp = N*0.7;
nbTest = N-nbApp;
nbVar = size(heart,2)-1; % On prend le nombre de variable d'etude sans le resultat

%initialisation d un vecteur d indices au hasard de 270 valeurs
indicesMelanges = randperm(N);
indicesApprentissage = indicesMelanges(1:nbApp);
indicesTest = indicesMelanges(nbApp+1:end);

%%% Observations

malade = heart(indicesApprentissage,nbVar+1);

%%%
%%% PROBAS
%%%

indiceMalade = find(malade==2);
nombreMalade = length(indiceMalade);

indicePasMalade = find(malade==1);
nombrePasMalade = length(indicePasMalade);

%% malade
% proba d etre malade (k=2)
pMalade = nombreMalade/length(indicesApprentissage);
pPasMalade = 1-pMalade;

%%%
%%% Variables continues
%%%

pCondi = cell(nbVar,1); % Un vecteur avec une matrice pour chaque variable pour stocker les proba condi
bornes = cell(nbVar,1); % Un vecteur de matrice pour stocker les bornes des intervalles de chaque variable
discrTest = zeros(nbTest,nbVar); % On construit une matrice nulle pour toutes les variables de test.

% On recupere les limites de chaque variable
minX = min (heart,[],1);
maxX = max (heart,[],1);

taille = 5; % On veut 5 intervalles pour chaque variable

% On rajoute un intervalle pour inclure le max dans l'avant dernier intervalle, et ne pas le laisser tout seul
largeurX = (maxX-minX)/taille;
maxX = maxX + largeurX;

varC = [1 4 5 8 10]; % On prend les numeros des variables continues

% On calcule les proba condi de chaque variable, on en profite pour initialiser les echantillons de test
for i=varC
    bornes{i} = linspace(minX(i),maxX(i),taille+1); % On recupere les bornes en sachant qu'on a ajoute un intervalle
    pCondi{i} = zeros(taille,2); % On initialise a zero, premiere colonne pour pas malade et 2eme pour malade
    
    % Proba de la variable sachant pas malade
    temp = histc(heart(indicePasMalade,i),bornes{i}); % On compte les sujets pas malades dans chaque intervalle de notre variable
    pCondi{i}(:,1) = temp(1:end-1)/nombrePasMalade; % On enleve le dernier nombre qui est nul (intervalle vide)
    
    % Proba de la variable sachant malade
    temp = histc(heart(indiceMalade,i),bornes{i}); % On compte les sujets pas malades dans chaque intervalle de notre variable
    pCondi{i}(:,2) = temp(1:end-1)/nombreMalade; % On enleve le dernier nombre qui est nul (intervalle vide)
        
    % Initialisation des echantillons de test
    % Dans discrTest, on recupere la matrice heart avec pour chaque valeur, son numero d'intervalle dans le decoupage selon les bornes
    [temp discrTest(:,i)] = histc(heart(indicesTest,i),bornes{i});
end

%%%
%%% Variables continues
%%%

varD = setdiff(1:nbVar,varC); % On recupere les numeros de colonne des variables discretes

% Pour chacune des variables discretes
for i=varD
    % on prend les elements qui apparaissent au moins une fois
    bornes{i} = unique(heart(:,i));
    
    % Nombre de valeurs differentes prise par la variable
    taille = length(bornes{i});
    
    pCondi{i} = zeros(taille,2); % On initialise a zero, premiere colonne pour pas malade et 2eme pour malade
    
    % On calcule directement car on a pas des intervalles mais des valeurs discretes (diagramme a batons)
    pCondi{i}(:,1) = histc(heart(indicePasMalade,i),bornes{i})/nombrePasMalade;
    pCondi{i}(:,2) = histc(heart(indiceMalade,i),bornes{i})/nombreMalade;
    
    % Initialisation des echantillons de test
    % Dans discrTest, on recupere la matrice heart avec pour chaque valeur, son numero de valeur dans le decoupage selon les valeurs
    [temp discrTest(:,i)] = histc(heart(indicesTest,i),bornes{i});
end

% Affichage des probabilites conditionnelles
% celldisp(pCondi);

%%%
%%% Classification de Bayes
%%%

% On cree une matrice de hauteur nbTest, de largeur 2 (pasMalade,Malade) avec la probabilite d'etre malade ou pas comme valeur
pTest = repmat([pPasMalade pMalade],nbTest,1);

% Pour chaque variable, on prend les proba condi en sachant l'intervalle pour chaque individu
% On recupere p(malade)*p(variableAppartientAIntervalle / malade ), 
% c'est a dire la proba d'appartenir a tel intervalle pour une variable ET d'etre malade ou pas
for i=1:nbVar
    pTest = pTest.*pCondi{i}(discrTest(:,i),:);
end

% On recupere les maximums sur chaque ligne (application de la regle de Bayes, max a posteriori) 
% et on garde le numero de la colonne dans classesBayes (1=>PasMalade, 2=> Malade)
[temp classesBayes] = max(pTest,[],2);

% On recupere les resultats reels, savoir si la personne est vraiment malade ou pas
classesReelles = heart(indicesTest,nbVar+1);

% Nombres de personnes bien diagnostiquees
nbBienDiagnostiquees = length (find(classesReelles == classesBayes));
% On affiche le resultat
fprintf('On a bien diagnostique %2.2f%% des personnes.\n',nbBienDiagnostiquees/nbTest*100);

% Tableau des totaux
tableau = zeros(2,2);

% On construit notre tableau incrementant selon les resultat, qui correspondent aux indices de notre tableau
% tableau (1,1) => diagnostique pas malade et pas malade
% tableau (1,2) => diagnostique pas malade et malade
% tableau (2,1) => diagnostique malade et pas malade
% tableau (2,2) => diagnostique malade et malade
for i=1:nbTest
    tableau(classesBayes(i),classesReelles(i)) = tableau(classesBayes(i),classesReelles(i))+1;
end

tableau
